/*
 * Telapathy Inspector - A Telepathy client which exposes Telepathy interfaces.
 *                       Meant to inspect and/or test connection managers.
 * 
 * ti-dlg-request-channel.c:
 * DlgRequestChannel - Request Channel Dialog
 * 
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
 * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "ti-dlg-request-channel.h"
#include "ti-config.h"

#include <glade/glade.h>

G_DEFINE_TYPE (TIDlgRequestChannel, ti_dlg_request_channel, G_TYPE_OBJECT);

/**
 * Instance private data.
 */
struct _TIDlgRequestChannelPrivate {
    gboolean disposed;

    GtkWindow* parent;

    GladeXML* glade_xml;

    GtkWidget* dialog;

    GtkComboBox* combo_type;
    GtkComboBox* combo_handle_type;
    GtkEntry* entry_handle;
    GtkCheckButton* check_suppress;
};
typedef struct _TIDlgRequestChannelPrivate TIDlgRequestChannelPrivate;

#define TI_DLG_REQUEST_CHANNEL_GET_PRIVATE(object)  (G_TYPE_INSTANCE_GET_PRIVATE ((object), TI_TYPE_DLG_REQUEST_CHANNEL, TIDlgRequestChannelPrivate))

/* Function prototypes */
static void _button_ok_clicked (GtkButton *button, gpointer user_data);
static void _button_cancel_clicked (GtkButton *button, gpointer user_data);

/**
 * Drop all references to other objects.
 */
static void
ti_dlg_request_channel_dispose (GObject *object)
{
    TIDlgRequestChannel *dlg_request_channel = TI_DLG_REQUEST_CHANNEL (object);
    TIDlgRequestChannelPrivate *priv = TI_DLG_REQUEST_CHANNEL_GET_PRIVATE (dlg_request_channel);

    if (priv->disposed) {
        return;
    } else {
        priv->disposed = TRUE;
    }

    if (priv->glade_xml != NULL) {
        g_object_unref (priv->glade_xml);
        priv->glade_xml = NULL;
    }

    G_OBJECT_CLASS (ti_dlg_request_channel_parent_class)->dispose (object);
}

/**
 * Class initialization.
 */
static void
ti_dlg_request_channel_class_init (TIDlgRequestChannelClass *ti_dlg_request_channel_class)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_dlg_request_channel_class);

	/* override base object methods */ 
	gobject_class->dispose = ti_dlg_request_channel_dispose;
		
	/* Add private */
	g_type_class_add_private (ti_dlg_request_channel_class, sizeof (TIDlgRequestChannelPrivate));
}

/**
 * Instance initialization.
 */
static void
ti_dlg_request_channel_init (TIDlgRequestChannel *ti_dlg_request_channel)
{
    TIDlgRequestChannelPrivate *priv = TI_DLG_REQUEST_CHANNEL_GET_PRIVATE (ti_dlg_request_channel);

    priv->disposed = FALSE;
}

/**
 * Returns a new instance.
 */
TIDlgRequestChannel*
ti_dlg_request_channel_new (GtkWindow* parent) 
{
    TIDlgRequestChannel* dialog = NULL;
    TIDlgRequestChannelPrivate* priv = NULL;
    gchar* glade_file_path = NULL;

	dialog = g_object_new (TI_TYPE_DLG_REQUEST_CHANNEL, NULL);
    priv = TI_DLG_REQUEST_CHANNEL_GET_PRIVATE (dialog);

    priv->parent = parent;

    glade_file_path = g_strdup_printf ("%s%s", TI_DATA_DIR_PREFIX, "dlg-request-channel.xml");
    priv->glade_xml = glade_xml_new (glade_file_path, NULL, NULL);
    if (priv->glade_xml == NULL)
    {
        g_critical ("Error loading glade file \"%s\".", glade_file_path);
        g_object_unref (dialog);
        dialog = NULL;
        goto CLEAN_UP;
    }

    priv->dialog = glade_xml_get_widget (priv->glade_xml, "dialog_request_channel");
    g_assert (GTK_IS_DIALOG (priv->dialog));

    priv->combo_type = GTK_COMBO_BOX (glade_xml_get_widget (priv->glade_xml, "combo_type"));
    g_assert (GTK_IS_COMBO_BOX (priv->combo_type));

    priv->combo_handle_type = GTK_COMBO_BOX (glade_xml_get_widget (priv->glade_xml, "combo_handle_type"));
    g_assert (GTK_IS_COMBO_BOX (priv->combo_handle_type));
    
    priv->entry_handle = GTK_ENTRY (glade_xml_get_widget (priv->glade_xml, "entry_handle"));
    g_assert (GTK_IS_ENTRY (priv->entry_handle));
    
    priv->check_suppress = GTK_CHECK_BUTTON (glade_xml_get_widget (priv->glade_xml, "checkbutton_suppress_handler"));
    g_assert (GTK_IS_CHECK_BUTTON (priv->check_suppress));

    gtk_window_set_transient_for (GTK_WINDOW (priv->dialog), GTK_WINDOW (priv->parent));

    glade_xml_signal_connect_data (priv->glade_xml, "button_ok_clicked", G_CALLBACK(_button_ok_clicked), priv->dialog);
    glade_xml_signal_connect_data (priv->glade_xml, "button_cancel_clicked", G_CALLBACK(_button_cancel_clicked), priv->dialog);

    CLEAN_UP:
    g_free (glade_file_path);

    return dialog;
}

/**
 * Run
 */
gboolean
ti_dlg_request_channel_run (TIDlgRequestChannel* dlg_request_channel,
                            gchar** type, guint* handle_type, guint* handle, gboolean* suppress_handler)
{
    TIDlgRequestChannelPrivate *priv = TI_DLG_REQUEST_CHANNEL_GET_PRIVATE (dlg_request_channel);
    gint result;
    gint active_handle_type;
    const gchar* handle_str;


    gtk_widget_show_all (priv->dialog);
    result = gtk_dialog_run (GTK_DIALOG (priv->dialog));
    gtk_widget_hide_all (priv->dialog);

    // Get type
    *type = gtk_combo_box_get_active_text (priv->combo_type);

    // Get handle type
    active_handle_type = gtk_combo_box_get_active (priv->combo_handle_type);
    *handle_type = (guint) active_handle_type;

    // Get handle
    handle_str = gtk_entry_get_text (priv->entry_handle);
    if (sscanf (handle_str, "%u", handle) != 1)
    {
        *handle = 0;
        g_warning ("Not an unsigned integer: \"%s\"", handle_str);
    }
    
    // Get suppress handler
    *suppress_handler = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_suppress));

    return result == GTK_RESPONSE_OK;
}

/**
 * Called when OK button is clicked.
 */
static void
_button_ok_clicked (GtkButton *button, gpointer user_data)
{
    GtkDialog* dialog = GTK_DIALOG (user_data);
    gtk_dialog_response (dialog, GTK_RESPONSE_OK);
}

/**
 * Called when "Cancel" button is clicked.
 */
static void
_button_cancel_clicked (GtkButton *button, gpointer user_data)
{
    GtkDialog* dialog = GTK_DIALOG (user_data);
    gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
}
